﻿package  {
	
import mx.containers.utilityClasses.BoxLayout;
import mx.containers.utilityClasses.Flex;
import mx.controls.scrollClasses.ScrollBar;
import mx.core.EdgeMetrics;
import mx.core.IUIComponent;
import mx.core.mx_internal;
import mx.core.ScrollPolicy;

import org.flashdevelop.utils.FlashConnect;

use namespace mx_internal;

	/**
	* ...
	* @author Default
	*/
	public class CoverFlowLayout extends BoxLayout {
		
		public function CoverFlowLayout() {
			
		}
		
		/**
		 *  @private
		 *  Lay out children as per Box layout rules.
		 */
		override public function updateDisplayList(unscaledWidth:Number,
												   unscaledHeight:Number):void
		{
			var target:RepeaterHBox = RepeaterHBox(super.target);
			
			var n:int = target.numChildren;
			if (n == 0)
				return;

			var vm:EdgeMetrics = target.viewMetricsAndPadding;

			var paddingLeft:Number = target.getStyle("paddingLeft");
			var paddingTop:Number = target.getStyle("paddingTop");

			var horizontalAlign:Number = getHorizontalAlignValue();
			var verticalAlign:Number = getVerticalAlignValue();

			var mw:Number = target.scaleX > 0 && target.scaleX != 1 ?
							target.minWidth / Math.abs(target.scaleX) :
							target.minWidth;
			var mh:Number = target.scaleY > 0 && target.scaleY != 1 ?
							target.minHeight / Math.abs(target.scaleY) :
							target.minHeight;

			var w:Number = Math.max(unscaledWidth, mw) - vm.right - vm.left;
			var h:Number = Math.max(unscaledHeight, mh) - vm.bottom - vm.top;

			var horizontalScrollBar:ScrollBar = target.horizontalScrollBar;
			var verticalScrollBar:ScrollBar = target.verticalScrollBar;

			var gap:Number;
			var numChildrenWithOwnSpace:int;
			var excessSpace:Number;
			var top:Number;
			var left:Number;
			var i:int;
			var obj:IUIComponent;

			if (n == 1)
			{
				// This is an optimized code path for the case where there's one
				// child. This case is especially common when the Box is really
				// a GridItem. This code path works for both horizontal and
				// vertical layout.

				var child:IUIComponent = IUIComponent(target.getChildAt(0));

				var percentWidth:Number = child.percentWidth;
				var percentHeight:Number = child.percentHeight;

				var width:Number;
				if (percentWidth)
				{
					width = Math.max(child.minWidth,
						Math.min(child.maxWidth,
						((percentWidth >= 100) ? w : (w * percentWidth / 100))));
				}
				else
				{
					width = child.getExplicitOrMeasuredWidth();
				}

				var height:Number
				if (percentHeight)
				{
					height = Math.max(child.minHeight,
						Math.min(child.maxHeight,
						((percentHeight >= 100) ? h : (h * percentHeight / 100))));
				}
				else
				{
					height = child.getExplicitOrMeasuredHeight();
				}

				// if scaled and zoom is playing, best to let the sizes be non-integer
				if (child.scaleX == 1 && child.scaleY == 1)
					child.setActualSize(Math.floor(width), Math.floor(height));
				else
					child.setActualSize(width, height);

				// Ignore scrollbar sizes for child alignment purpose.
				if (verticalScrollBar != null &&
					target.verticalScrollPolicy == ScrollPolicy.AUTO)
				{
					w += verticalScrollBar.minWidth;
				}
				if (horizontalScrollBar != null &&
					target.horizontalScrollPolicy == ScrollPolicy.AUTO)
				{
					h += horizontalScrollBar.minHeight;
				}

				// Use the child's width and height because a Resize effect might
				// have changed the child's dimensions. Bug 146158.
				left = (w - child.width) * horizontalAlign + paddingLeft;
				top = (h - child.height) * verticalAlign + paddingTop;

				child.move(Math.floor(left), Math.floor(top));
			}

			else  // HBOX
			{
				gap = target.getStyle("horizontalGap");

				numChildrenWithOwnSpace = n;
				for (i = 0; i < n; i++)
				{
					if (!IUIComponent(target.getChildAt(i)).includeInLayout)
						numChildrenWithOwnSpace--;
				}

				// stretch everything as needed including heights
				excessSpace = Flex.flexChildWidthsProportionally(
					target, w - (numChildrenWithOwnSpace - 1) * gap, h);

				// Ignore scrollbar sizes for child alignment purpose.
				if (horizontalScrollBar != null &&
					target.horizontalScrollPolicy == ScrollPolicy.AUTO)
				{
					h += horizontalScrollBar.minHeight;
				}
				if (verticalScrollBar != null &&
					target.verticalScrollPolicy == ScrollPolicy.AUTO)
				{
					excessSpace += verticalScrollBar.minWidth;
				}

				left = paddingLeft + excessSpace * horizontalAlign;

				for (i = 0; i < n; i++)
				{
					obj = IUIComponent(target.repeaterArray[i]);
					top = (h - obj.height) * verticalAlign + paddingTop;
					obj.move(Math.floor(left), Math.floor(top));
					if (obj.includeInLayout)
						left += obj.width + gap;
				}
			}
		}
	}
	
}